# -*- coding: utf-8 -*-

"""Unit tests for oracle database module."""

import os
import unittest
import pandas
from sqlalchemy.exc import DatabaseError
from etlstat.database.oracle import Oracle

os.environ['NLS_LANG'] = '.AL32UTF8'


class TestSqlLoader(unittest.TestCase):
    """Testing methods for Oracle class."""

    user = 'test'
    password = 'password'
    host = 'localhost'
    port = '1521'
    service_name = 'xe'
    conn_params = [user, password, host, port, service_name]
    output_path = os.getcwd() + '/etlstat/database/test/'
    os_path = '/usr/local/bin:/usr/bin:/bin:' \
        '/opt/oracle/instantclient_18_3'
    os_ld_library_path = '/opt/oracle/instantclient_18_3'

    @classmethod
    def setUpClass(cls):
        """Set up test variables."""
        user = 'system'
        password = 'oracle'
        host = 'localhost'
        port = '1521'
        service_name = 'xe'
        conn_params = [user, password, host, port, service_name]
        ora_conn = Oracle(*conn_params)
        try:
            sql = "CREATE USER test IDENTIFIED BY password " \
                    "DEFAULT TABLESPACE USERS TEMPORARY TABLESPACE TEMP"
            ora_conn.execute(sql)
        except DatabaseError as dbe:
            print(str(dbe))
        sql = "ALTER USER test QUOTA UNLIMITED ON USERS"
        ora_conn.execute(sql)
        sql = "GRANT CONNECT, RESOURCE TO test"
        ora_conn.execute(sql)

    def test_insert(self):
        """Check if a bulk insert with sql loader is correctly executed."""
        table_name = 'px_01001'
        ora_conn = Oracle(*self.conn_params)
        sql = "CREATE TABLE {0} (id INTEGER, tipo_indicador " \
            "VARCHAR(100), nivel_educativo VARCHAR(100), valor NUMBER)". \
            format(table_name)
        try:
            ora_conn.execute(sql)
        except DatabaseError as dbe:
            print(str(dbe))
        px_01001 = ora_conn.get_table(table_name)
        self.assertTrue(px_01001.exists)
        source_file = self.output_path + 'px_01001.csv'
        data_file = self.output_path + 'px_01001.dat'
        control_file = self.output_path + 'px_01001.ctl'
        log_file = self.output_path + 'px_01001.log'
        bad_file = self.output_path + 'px_01001.bad'
        data_columns = ['id', 'tipo_indicador', 'nivel_educativo', 'valor']
        table_def = pandas.DataFrame(columns=data_columns)
        table_def['id'] = table_def['id'].astype(int)
        table_def['tipo_indicador'] = table_def['tipo_indicador'].astype(str)
        table_def['nivel_educativo'] = table_def['nivel_educativo'].astype(str)
        table_def['valor'] = table_def['valor'].astype(float)
        table_def.name = table_name
        data = pandas.read_csv(source_file, header=0, sep=';', encoding='utf8')
        data.name = table_name
        ora_conn.insert(
            *self.conn_params,
            schema=self.conn_params[0],
            table=data,
            output_path=self.output_path,
            os_path=self.os_path,
            os_ld_library_path=self.os_ld_library_path
        )
        self.assertTrue(os.path.isfile(data_file))
        os.remove(data_file)
        self.assertTrue(os.path.isfile(control_file))
        os.remove(control_file)
        self.assertTrue(os.path.isfile(log_file))
        os.remove(log_file)
        self.assertFalse(os.path.isfile(bad_file))


if __name__ == '__main__':
    unittest.main()



# coding: utf-8
"""
This module manages MySQL primitives.

    Date:
        27/11/2018

    Author:
        emm13775

    Version:
        0.1

    Notes:

"""

import logging
from sqlalchemy import create_engine, text, select, func, MetaData, Table
from sqlalchemy.exc import DatabaseError
import pandas as pd
import numpy as np
import d6tstack

logging.basicConfig(level=logging.INFO)
LOGGER = logging.getLogger(__name__)


class MySQL:
    """
    Manage connections to MySQL databases.

    MySQL class offers some helper methods that encapsulate primitive logic
    to interactuate with the database: insert/upsert, execute, drop, etc.
    Some primitives are not included because they can be handled more
    properly with sqlalchemy.

    """

    def __init__(self, *conn_params):
        """
        Initialize the database connection and other relevant data.

        Args:
            *conn_params: list with the following connection parameters:
                user(string): database user to connect to the schema.
                password(string): database password of the user.
                host(string): database management system host.
                port(string): tcp port where the database is listening.
                database(string): schema or database name.

        """
        # connection string in sqlalchemy format
        self.conn_string = f'''mysql+mysqlconnector://{conn_params[0]}:''' + \
            f'''{conn_params[1]}@{conn_params[2]}:''' + \
            f'''{str(conn_params[3])}/{conn_params[4]}'''
        self.engine = create_engine(self.conn_string)
        self.database = conn_params[4]

    def get_table(self, table_name, schema=None):
        """
        Get a database table into a sqlalchemy Table object.

        Args:
            table_name(string): name of the database table to map.
            schema(string): name of the schema to which the table belongs.
                        Defaults to the selected database in the
                        connection.
        Returns:
            table(Table): sqlalchemy Table object referencing the specified
                            database table.

        """
        meta = MetaData(bind=self.engine,
                        schema=schema if schema else self.database)
        return Table(table_name, meta, autoload=True,
                     autoload_with=self.engine)

    def execute(self, sql, **kwargs):
        """
        Execute a DDL or DML SQL statement.

        Args:
            sql: SQL statement
        Returns:
            result_set(Dataframe):

        """
        connection = self.engine.connect()
        trans = connection.begin()
        result_set = pd.DataFrame()
        try:
            result = connection.execute(text(sql), **kwargs)
            trans.commit()
            if result.returns_rows:
                result_set = pd.DataFrame(result.fetchall())
                result_set.columns = result.keys()
                LOGGER.info('Number of returned rows: %s',
                            str(len(result_set.index)))
        except DatabaseError as db_error:
            LOGGER.error(db_error)
            raise
        finally:
            connection.close()
        return result_set

    def drop(self, table_name):
        """
        Drop a table from the database.

        Args:
          table_name(str): name of the table to drop.

        Returns: nothing.

        """
        db_table = self.get_table(table_name)
        db_table.drop(self.engine, checkfirst=True)
        LOGGER.info('Table %s successfully dropped.', table_name)

        # Placeholders can only represent VALUES. You cannot use them for
        # sql keywords/identifiers.

    def insert(self, data_table, primary_key='id'):
        """
        Insert a dataframe into a table.

        Converts the dataframe to CSV format and uses odo
        bulk load functionality.

        Args:
          data_table(Dataframe): dataframe with the data to load.
          primary_key(string): primary key column name. Defaults to 'id'.

        Returns:
          db_table(Table): sqlalchemy table mapping the table with the inserted
                           records.

        """
        connection = self.engine.connect()
        db_table = Table()
        if isinstance(data_table, pd.DataFrame):
            aux = data_table.replace(np.NaN, "\\N")
        else:
            raise TypeError("data_table must be a DataFrame.")
        try:
            d6tstack.utils.pd_to_mysql(aux, self.conn_string,
                                       data_table.name,
                                       if_exists='replace')
            db_table = self.get_table(data_table.name)
            row_count = connection.engine.scalar(
                select([func.count('*')]).select_from(db_table)
            )
            connection.execute(f'''alter table {data_table.name}
                                add primary key({primary_key})''')
            LOGGER.info('Number of inserted rows: %s', str(row_count))
        except Exception as exception:
            LOGGER.error(exception)
            raise
        finally:
            connection.close()
        return db_table

    def upsert(self, tmp_data, table_name, sql, primary_key='id', rm_tmp=True):
        """
        Update/insert a dataframe into a table.

        Converts the dataframe to CSV format, and uses odo bulk load
        functionality to load it to a temporary table and then executing
        a raw update or update/insert query from a text file which
        extracts records from the temporary table and loads them into the
        definitive one.

        Args:
            tmp_data(Dataframe): dataframe with the data to load in a
                                    temporary table.
            table_name(String): name of the table to be
                                updated/inserted to.
            sql(string): string with the SQL update/insert query.
            primary_key(string): primary key column name. Defaults to 'id'.
            rm_tmp(Boolean): Defauls to True. Determines if the temporary
                                table should be dropped (expected behaviour)
                                or not (for debugging purposes).

        Returns:
            db_table(Table): sqlalchemy table mapping the table with the
                                inserted/updated records.

        """
        connection = self.engine.connect()
        tmp_table = Table()
        if isinstance(tmp_data, pd.DataFrame):
            aux = tmp_data.replace(np.NaN, "\\N")
        else:
            raise TypeError("tmp_table must be a DataFrame.")
        try:
            d6tstack.utils.pd_to_mysql(aux, self.conn_string,
                                       tmp_data.name,
                                       if_exists='replace')
            tmp_table = self.get_table(tmp_data.name)
            connection.execute(f'''alter table {tmp_data.name}
                               add primary key({primary_key})''')
            tmp_row_count = connection.engine.scalar(
                select([func.count('*')]).select_from(tmp_table)
            )
            LOGGER.info('Number of temp table rows: %s', str(tmp_row_count))
            connection.execute(sql)  # update/insert query
            if rm_tmp:
                self.drop(tmp_table.name)  # remove temporary table
            db_table = self.get_table(table_name)
            row_count = connection.engine.scalar(
                select([func.count('*')]).select_from(db_table)
            )
            LOGGER.info('Number of rows in the updated table: %s',
                        str(row_count))
        except Exception as exception:
            LOGGER.exception(exception)
            raise
        finally:
            connection.close()
        return db_table



# coding: utf-8
"""
This module manages Oracle primitives.

    Date:
        23/01/2018

    Author:
        lla11358

    Version:
        0.1

    Notes:

"""

import csv
import logging
import os
import shlex
import subprocess
import pandas
from sqlalchemy import create_engine, text, MetaData, Table
from sqlalchemy.exc import DatabaseError

logging.basicConfig(level=logging.INFO)
LOGGER = logging.getLogger(__name__)


class Oracle:
    """
    Manage connections to Oracle databases.

        Oracle class offers some helper methods that encapsulate primitive
        logic to interactuate with the database: insert/upsert, execute,
        drop, etc.
        Some primitives are not included because they can be handled more
        properly with sqlalchemy.

    """

    def __init__(self, *conn_params, encoding='utf8'):
        """
        Initialize the database connection and other relevant data.

            Args:
                *conn_params: list of the following connection parameters:
                    user(string): database user to connect to the schema.
                    password(string): database password of the user.
                    host(string): database management system host.
                    port(string): tcp port where the database is listening.
                    service_name(string): Oracle instance name.
                encoding (string): Charset encoding.
        """
        # connection string in sqlalchemy format
        self.conn_string = f'''oracle+cx_oracle://{conn_params[0]}:''' + \
            f'''{conn_params[1]}@{conn_params[2]}:''' + \
            f'''{conn_params[3]}/{conn_params[4]}'''
        self.engine = create_engine(self.conn_string,
                                    encoding=encoding,
                                    coerce_to_unicode=True,
                                    coerce_to_decimal=False)
        self.schema = conn_params[0]

    def get_table(self, table_name, schema=None):
        """
        Get a database table into a sqlalchemy Table object.

            Args:
                table_name(string): name of the database table to map.
                schema(string): name of the schema to which the table belongs.
                    Defaults to the selected database in the connection.
            Returns:
                table(Table): sqlalchemy Table object referencing the specified
                    database table.

        """
        meta = MetaData(bind=self.engine,
                        schema=schema if schema else self.schema)
        return Table(table_name, meta, autoload=True,
                     autoload_with=self.engine)

    def execute(self, sql, **kwargs):
        """
        Execute a DDL or DML SQL statement.

            Args:
                sql (string): SQL statement
                kwargs (dictionary): optional statement named parameters
            Returns:
                result_set(Dataframe):

        """
        connection = self.engine.connect()
        trans = connection.begin()
        result_set = pandas.DataFrame()
        try:
            result = connection.execute(text(sql), **kwargs)
            trans.commit()
            if result.returns_rows:
                result_set = pandas.DataFrame(result.fetchall())
                result_set.columns = result.keys()
                LOGGER.info('Number of returned rows: %s',
                            str(len(result_set.index)))
        except DatabaseError as db_error:
            LOGGER.error(db_error)
            raise
        finally:
            connection.close()
        return result_set

    def drop(self, table_name, schema=None):
        """
        Drop a table from the database.

        Args:
          table_name(str): name of the table to drop.

        Returns: nothing.

        """
        if not schema:
            schema = self.schema
        db_table = self.get_table(table_name, schema)
        db_table.drop(self.engine, checkfirst=True)
        LOGGER.info('Table %s.%s successfully dropped.', schema, table_name)

        # Placeholders can only represent VALUES. You cannot use them for
        # sql keywords/identifiers.

    @staticmethod
    def insert(
            *conn_params,
            schema,
            table,
            output_path,
            os_path,
            os_ld_library_path,
            mode="APPEND"
    ):
        """
        Load a dataframe into a table via Oracle SQL Loader.

        Extracts field names and data values from a Pandas DataFrame.
        Requires Oracle Instant Client and Tools installed in the workstation.
        Destination table must exist in the database.
        Usage of SQL Loader:

            sqlldr <user>/<password> control=<control_file> [log=<log_file>]
            [bad=bad_file]

        Args:
            *conn_params: list of the following connection parameters:
                    user(string): database user to connect to the schema.
                    password(string): database password of the user.
                    host(string): database management system host.
                    port(string): tcp port where the database is listening.
                    service_name(string): Oracle instance name.
            schema (str): database schema
            table (DataFrame): dataframe with the same name and column
                labels as the table in which it's going to be loaded.
                It must be filled with data rows.
            output_path (str): path for output data files
            os_path (str): PATH environment variable
            os_ld_library_path (str): LD_LIBRARY_PATH environment variable
            mode (str): insertion mode: APPEND | REPLACE | TRUNCATE

        """
        columns = ",".join(table.columns.values.tolist())

        # control file
        ctl_file = open(f'''{output_path}{table.name}.ctl''',
                        mode='w',
                        encoding='utf8')
        ctl_header = f"""LOAD DATA
                     CHARACTERSET UTF8
                     INFILE '{output_path}{table.name}.dat'
                     {mode}
                     INTO TABLE {schema}.{table.name}
                     FIELDS TERMINATED BY ';' OPTIONALLY ENCLOSED BY '\"'
                     TRAILING NULLCOLS
                     ({columns})"""

        ctl_file.write(ctl_header)
        ctl_file.close()

        # data file
        table.to_csv(f'''{output_path}{table.name}.dat''',
                     sep=';',
                     header=False,
                     index=False,
                     doublequote=True,
                     quoting=csv.QUOTE_NONNUMERIC,
                     encoding='utf-8'
                     )

        # set environment variables
        env = os.environ.copy()
        env['PATH'] = os_path
        env['LD_LIBRARY_PATH'] = os_ld_library_path
        # generate SQL Loader arguments
        os_command = f"""sqlldr {conn_params[0]}/{conn_params[1]}""" + \
            f"""@{conn_params[2]}:{conn_params[3]}/{conn_params[4]} """ + \
            f"""control='{output_path}{table.name}.ctl' """ + \
            f"""log='{output_path}{table.name}.log' """ + \
            f"""bad='{output_path}{table.name}.bad'"""
        args = shlex.split(os_command)
        # execution of Oracle SQL Loader
        try:
            subprocess.Popen(args, env=env)
        except subprocess.SubprocessError as sproc_error:
            LOGGER.error(sproc_error)



# coding: utf-8
"""Text parsing related utils.

This module contains several text related utils which have been considered
to be reusable in different ETL projects.

Date: 11/12/2018
Author: emm13775
Version: 0.1
Notes:

"""

import os
import re
import logging
from unidecode import unidecode
from etlstat.extractor.extractor import xml, csv

# log configuration
logging.basicConfig(level=logging.INFO)
LOGGER = logging.getLogger(__name__)


def parse_df_columns(data):
    """Parse columns in a dataframe.

    Converts capital letters into lowercase letters, white spaces into low
    bar and removes accent marks.

    Args:
        data(Dataframe): pandas dataframe with the columns to parse.
    Returns:
        data(Dataframe): pandas dataframe with the parsed columns.

    """
    parsed_columns = []
    for column in data.columns:
        to_replace = re.sub(r'[ ]{3,}', ' ',
                            unidecode(column.lower().replace(',', '')))
        parsed_columns.append(re.sub(r'[ ]{2,}', '',
                                     to_replace).replace(' ', '_'))
    data.columns = parsed_columns
    return data


def replace_urls_in_xml(tree, urls_df, output_file):
    """Replace URLs in Pentaho Data Integration (Kettlt) xml files.

    First thought for PC-Axis URLs, but it can be used with any generic URL.
    Produces a xml file which is written to the output file path parameter.


    Args:
        tree (ElementTree): ElementTree object with the parsed xml file.
        urls_df (DataFrame): pandas DataFrame with a mapping between the
        old and the new urls. This dataframe MUST HAVE two columns:
        old_urls and new_urls, containing the URLs to replace and the new
        ones respectively.
        output_file (str): output file path including file name.

    """
    urls_dict = dict(zip(urls_df.old_urls, urls_df.new_urls))
    root = tree.getroot()
    entries = root.find('entries')

    urls = []
    found = {}

    for entry in entries:
        url = entry.find('url')
        if url is not None:
            splitted = re.split('&url=', url.text)
            url_found = urls_dict.get(splitted[1], None)
            if url_found:
                LOGGER.info("URL found: %s", str(url_found))
                found[splitted[1]] = found.get(splitted[1], 0) + 1
                url.text = splitted[0] + '&url=' + urls_dict[splitted[1]]
                urls.append(url.text)
            else:
                LOGGER.info("URL NOT found: %s", splitted[1])
    LOGGER.info("Found total unique URLS:  %s", str(len(found.keys())))
    tree.write(output_file)


def bulk_replace_url_in_xml(input_data_path, output_data_path=None):
    """Massively replaces URLs in a bunch of PDI (Kettle) XML files.

    Args:
        input_data_path: path to the directory which contains the URL mappings
        in a two column CSV file and the PDI (Kettle) XML Files.
        output_data_path: path to the directory which contains the parsed XML
        files. If not specified, defaults to None and then an 'output'
        directory is created inside of 'input_data_path' directory, so that
        the results can be written there.
    """
    job_dirs = [directory[0] +
                '/' for directory in os.walk(input_data_path)][1:]
    for job_dir in job_dirs:
        if os.listdir(job_dir):  # check if directory is empty
            output_dir = output_data_path if output_data_path \
                else input_data_path + 'output/'
            if not os.path.exists(output_dir):
                os.makedirs(output_dir)
            jobs = xml(job_dir)
            mappings = csv(job_dir, sep=',')
            if mappings:
                LOGGER.info("Using mapping file: %s",
                            str(next(iter(mappings))))
                for key, value in jobs.items():
                    replace_urls_in_xml(value,
                                        mappings[next(iter(mappings))],
                                        output_dir + key)



# -*- coding: utf-8 -*-

"""Extractor module: collection of functions for massively data extraction.

This module offers several helping methods and functions to ease massively
data extraction from different file formats. Support for XLS, CSV, PX, XML, SQL
and TXT positional files is included. Extractor allows to read every file in
a directory and creates a dictionary keeping their file names as keys and
their data as values. If possible, these data are parsed into a pandas
dataframe.

"""

import fnmatch
import os
from contextlib import ExitStack
import defusedxml.ElementTree as ET
import xlrd
import numpy as np
import pandas as pd
import Levenshtein
from pyaxis import pyaxis


def match_data_format(data_path, data_extension,
                      format_path, format_extension):
    """Match format files(csv) with data files (txt) for positional files.

    Args:
        data_path (str): directory containing data files.
        data_extension (str): standard for data filenames extensions.
        format_path (str): directory containing format files.
        format_extension (str): standard for format filenames extensions.

    Returns:
        dict: Data filenames (TXT) as KEYS and format filenames (CSV) as
              VALUES.

    """
    assignation_map = {}
    # Contains data filenames
    data_list = []
    os.chdir(data_path)
    for file in os.listdir('.'):
        # Finds every data file in a directory
        if fnmatch.fnmatch(file, data_extension):
            data_list.append(file)
    if not data_list:
        raise FileNotFoundError("No data files found in data directory")
    data_list = set(data_list)
    data_list = list(data_list)

    # Contains format filenames
    format_list = []
    os.chdir(format_path)
    for file in os.listdir('.'):
        # Finds every format file in a directory
        if fnmatch.fnmatch(file, format_extension):
            format_list.append(file)
    if not format_list:
        raise FileNotFoundError("No format files found in format directory")
    format_list = set(format_list)
    format_list = list(format_list)

    for item in data_list:
        max_similarity = 0
        max_element = None
        for element in format_list:
            similarity = Levenshtein.ratio(item, element)
            if similarity > max_similarity:
                max_similarity = similarity
                max_element = element
        assignation_map[item] = max_element
        # logger.info("Matched data file: " + item + " with: " +
        #             assignation_map[item])
    return assignation_map


def xls(dir_path, sep=';', encoding='utf-8',
        data_extension='*.[xX][lL][sS]', na_values=None):
    """Massively read XLS files from a directory.

    Read excel files in a directory and generate a dict with xls names and
    sheetnames as keys and dataframes with the data as values.

    Args:
        dir_path (str): directory containing Excel files.
        sep (str): field separator.
        encoding (str): file encoding.
        data_extension (str): standard for data filenames extensions.
        na_values (scalar, str, list-like, or dict) : Additional strings to
                                                      recognize as NA/NaN.

    Returns:
        dict: Excel name and sheet_names as KEYS and dataframe as VALUE.

    """
    os.chdir(dir_path)
    data = {}
    for excel in os.listdir('.'):
        if fnmatch.fnmatch(excel, data_extension):
            data[excel] = dict()
            book = xlrd.open_workbook(dir_path + excel)
            for sheet in book.sheet_names():
                data[excel][sheet] = pd.read_excel(open(dir_path + excel,
                                                        'rb'),
                                                   encoding=encoding,
                                                   sep=sep,
                                                   sheet_name=sheet,
                                                   na_values=na_values
                                                   )
                data[excel][sheet].name = sheet
    return data


def csv(
        dir_path,
        data_extension='*.[cC][sS][vV]',
        dtype=None, encoding='utf-8',
        na_values=None,
        sep=';',
        skipinitialspace=False
):
    """Massively read CSV files from a directory.

    Read csv files in a directory and generate a dict with csv names as keys
    and dataframes as values.

    Args:
        dir_path (str): directory containing Csv files.
        data_extension (str): standard for data filenames extensions.
        dtype (str, dict): data type for data or columns.
        encoding (str): file encoding.
        na_values (scalar, str, list-like, or dict) : Additional strings to
                                                      recognize as NA/NaN.
        sep (str): field separator.
        skipinitialspace (bool): skip spaces after delimiter.

    Returns:
        dict: Csv name as KEY and dataframe as VALUE

    """
    os.chdir(dir_path)
    data = {}
    for file in os.listdir('.'):
        if fnmatch.fnmatch(file, data_extension):
            data[file] = pd.read_csv(dir_path + file,
                                     dtype=dtype,
                                     encoding=encoding,
                                     na_values=na_values,
                                     sep=sep,
                                     skipinitialspace=skipinitialspace)
            data[file].name = file
    return data


def px(filename, sep=",", encoding='windows-1252'):
    """Massively read PC-Axis files from a directory.

    Read and convert PC-Axis files to dataframes from URIs listed in a CSV
    file.

    Args:
        filename (str): CSV FILE with uris file path (including file name).
        sep (str): field separator.
        encoding (str): file encoding.
    Returns:
        dict: file names as keys and dataframes as values.

    """
    uris = pd.read_csv(filename,
                       sep=sep,
                       encoding=encoding)
    data = {}
    uris['data'] = uris.apply(lambda row: pyaxis.parse(
        row['url'], encoding)['DATA'], axis=1)
    data = pd.Series(uris['data'].values, index=uris['id']).to_dict()
    return data


def txt(dir_path, sep=';', encoding='windows-1252',
        format_extension='*.[cC][sS][vV]', data_extension='*.[tT][xX][tT]',
        na_values=None, format_path=None):
    """Massively read positional text files from a directory.

    Read files in a directory, generate a correspondence between data and
    format files and return a dict (MICRODATA). It uses match_data_format() in
    order to generate the dataframe.

    Args:
        dir_path (str): directory containing data files.
        sep (str): field separator.
        encoding (str): file encoding.
        format_extension (str): standard for format name.
        data_extension (str): standard for data name.
        na_values (scalar, str, list-like, or dict) : Additional strings to
                                                      recognize as NA/NaN.
        format_path (str): directory containing format files.
                           Defaults to dir_path.


    Returns:
        dict: Name of data file as KEY and dataframe as VALUE.

    """
    conversion_map = {
        'STRING': str,
        'NUMBER': np.float32,
        'DECIMAL': np.float32,
        'INTEGER': np.float32
    }

    if not format_path:
        format_path = dir_path

    assignation_map = match_data_format(dir_path, data_extension, format_path,
                                        format_extension)
    for txt_file in assignation_map:
        conversion = dict()
        assignation_map[txt_file] = pd.read_csv(format_path +
                                                assignation_map[txt_file],
                                                sep=sep,
                                                encoding=encoding)
        for line in range(len(assignation_map[txt_file])):
            conversion[assignation_map[txt_file]['FIELD_NAME'][line]
                       ] = conversion_map[assignation_map[txt_file]
                                          ['DATA_TYPE'][line]]
        assignation_map[txt_file] = pd.read_fwf(dir_path + txt_file,
                                                widths=assignation_map
                                                [txt_file]['LENGTH'].tolist(),
                                                names=assignation_map[txt_file]
                                                ['FIELD_NAME'].tolist(),
                                                dtype=conversion,
                                                nwords=0,
                                                encoding=encoding,
                                                na_values=na_values)
        assignation_map[txt_file].name = txt_file
    return assignation_map


def xml(dir_path, pattern='*.[xXkK][mMtTjJ][lLrRbB]'):
    """Massively read XML files from a directory.

    Read files in a directory filtered by regEx and generate a dict with xml
    file names as keys and etree objects as values.

    Args:
        dir_path (str): directory containing XML files.
        pattern (str): regEx to filter data names.

    Returns:
        dict: XML name as KEY and etree object as VALUE.

    """
    xml_list = []
    os.chdir(dir_path)
    for file in os.listdir('.'):
        if fnmatch.fnmatch(file, pattern):
            xml_list.append(file)
    xml_list = set(xml_list)
    xml_list = list(xml_list)
    df_dict = dict.fromkeys(xml_list, '')

    for i in enumerate(xml_list):  # warning: enumerate returns tuples
        df_dict[xml_list[i[0]]] = ET.parse(dir_path + xml_list[i[0]])
    return df_dict


def sql(dir_path):
    """Massively read SQL files from a directory.

    Read every SQL file in a directory and generates a dict with the file
    names as keys and the content of the files as values.

    Args:
        dir_path (str): Path of DIR readed.

    Returns:
        dict: query name  as KEY and query as VALUE.

    """
    files = {}
    os.chdir(dir_path)
    with ExitStack() as context_manager:
        for filename in os.listdir('.'):
            if fnmatch.fnmatch(filename, '*.sql'):
                sql_file = context_manager.enter_context(open(filename, 'r'))
                files[filename.split(dir_path, 1)[-1][:-4]] = sql_file.read()
        context_manager.pop_all().close()
    return files



"""
This module includes helping classes and methods for common logging tasks.
"""

import logging


class LoggerMixin(object):
    """ Gets a logger for the current class. Classes must inherit from this
        in order to use self.logger without the need of declaring it in every
        class. Reference:
        https://easyaspython.com/mixins-for-fun-and-profit-cb9962760556
    """
    @property
    def logger(self):
        """
            Gets a logger for the current class.

            Args: None.

            Returns:
                logger: logger for the current class.
        """
        name = '.'.join([
            self.__module__,
            self.__class__.__name__
        ])
        return logging.getLogger(name)



# -*- coding: utf-8 -*-
from setuptools import setup, find_packages

setup(
    name='etlstat',
    version='0.3.1',
    author='Instituto Cántabro de Estadística',
    author_email='icane@cantabria.es',
    packages=find_packages(),
    url='https://github.com/icane/etlstat.git',
    license='Apache License 2.0',
    description='ETL utils for statistical offices data processing',
    long_description=open('README.rst').read(),
    install_requires=[
        'cx_Oracle',
        'mysql-connector>=2.1.4',
        'SQLAlchemy>=1.2.14',
        'd6tstack>=0.1.6',
        'Unidecode>=1.0.22',
        'xlrd>=1.1.0',
        'defusedxml>=0.5.0',
        'pyaxis>=0.1.0',
        'numpy>=1.15.4',
        'pandas>=0.23.4',
        'python_Levenshtein>=0.12.0'
    ],
    test_suite='extractor.test, database.test, text.test',
    keywords=['etl', 'icane', 'statistics', 'utils'],
    classifiers=[
          'Development Status :: 4 - Beta',
          'Environment :: Console',
          'Intended Audience :: Developers',
          'Intended Audience :: Science/Research',
          'License :: OSI Approved :: Apache Software License',
          'Operating System :: MacOS :: MacOS X',
          'Operating System :: Microsoft :: Windows',
          'Operating System :: POSIX',
          'Programming Language :: Python',
          'Topic :: Scientific/Engineering :: Information Analysis',
          'Topic :: Software Development :: Libraries'
          ],
)



"""
    Class to handle time logging

    Date:
        September 2017

    Authors:
        emm13775

    Version:
        0.1

    Notes:

"""

import time
import logging as log

log.basicConfig(level=log.INFO)
LOGGER = log.getLogger(__name__)


def timeit(method):
    """Execute a timer when a method starts.

    Used as annotation, allows to log the time a method or function takes to
    execute.
    """
    def timed(*args, **kw):
        ts = time.time()
        result = method(*args, **kw)
        te = time.time()
        LOGGER.info('[time] method: %r (%r, %r) %2.2f sec' %
                    (method.__name__, args, kw, te-ts))
        return result
    return timed
